//%Z% %I% %W% %G% %U% [%H% %T%]
/*
 * Change History:
 *
 * Reason         Version    Date       User id     Description
 * ----------------------------------------------------------------------------
 * F017049-19395	8.0		07/21/2010	shighbar	Add remote reader test on HPEL mBean via JMX.
 */

package com.ibm.sample.hpel;

import java.util.Properties;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.ObjectName;

import com.ibm.websphere.management.exception.ConfigServiceException;
import com.ibm.websphere.management.exception.ConnectorException;

public interface JmxClient {

	/**
	 * creates the AdminClient object via the factory.
	 */
	public  void createAdminClient() throws Exception;
	
	/**
	 * creates the AdminClient object via the factory using the properties passed in
	 * @param connectionProperties properties object that contains all the needed connection properties
	 */
	public  void createAdminClient(Properties connectionProperties) throws Exception;

	/**
	 * queries for MBeans matching a particular pattern
	 * @param QueryString String containing query information
	 * @param CkTooMany boolean to say if > 1 is an error, or OK
	 * @return array of ObjectNames that met query
	 */
	public  ObjectName[] Ck4MBean(String QueryString, boolean CkTooMany);

	/**
	 * lists Jmx MetaData for an Object. This is for debugging/doc purposes
	 * @param aObjNm Name of the object to list MetaData for
	 */
	public  void listMBeanInfo(ObjectName aObjNm);

	/**
	 * invokes a method on an MBean via JMX adminClient
	 * @param aObjName JMX ObjectName to invoke the method on
	 * @param ObjOpName name of MBean operation to invoke
	 * @param ObjParams array of Objects to be passed to the object
	 * @param ObjSignature array of Strings describing the parms
	 * @return java Object return from the Jmx call
	 */
	public  Object ObjInvoker(ObjectName aObjName, String ObjOpName,
			Object[] ObjParams, String[] ObjSignature) throws Exception ;

	/**
	 * gets an attribute from an object in JMX via adminClient.getAttribute call
	 * @param aObjNm ObjectName on which the attribute resides
	 * @param aAttrNm Name of the Attribute to retrieve
	 * @return java Object containing the value
	 */
	public  Object getAttribute(ObjectName aObjNm, String aAttrNm) throws Exception ;

	/**
	 * sets an attribute via a Jmx adminClient call
	 * @param aObjNm ObjectName on which the attribute resides
	 * @param aAttrNm new Attribute for this ObjectName
	 */
	public  void setAttribute(ObjectName aObjNm, Attribute aAttrNm) throws Exception ;

	/**
	 * retrieve the Admin Client object being used by the implementation.  This should only be used if the operations
	 * supported in the JmxClient interface are not sufficient
	 */
	public  com.ibm.websphere.management.AdminClient getAdminClient() ;

	//  Configuration work methods
	/**
	 * create a configuration service reference using the admin client
	 */
	public void createConfigService() ;
	
	/**
	 * resolve/get a list of configuration object names based on a type, and more specific key
	 * @param type type of object
	 * @param key key string
	 * @return a list of objects of that type within the scope specified, matching the key
	 */
	public ObjectName [] resolveConfigObject(String fullKey) ;
	
	/**
	 * get a list of configuration object names based on a type, key, and scope
	 * @param type type of object
	 * @param key key string
	 * @param scope object used to scope the search (can be null)
	 * @return a list of objects of that type within the scope specified, matching the key
	 */
	public ObjectName [] getConfigObject(String type, String key, ObjectName scope) ;

	/**
	 * get a configuration attribubute value from an object
	 * @param owningObject configuration object we are querying
	 * @param attrName the name of the attribute
	 * @return value of the attribute
	 */
	public Object getConfigAttribute(ObjectName owningObject, String attrName) ;

	/**
	 * set config attributes on an object
	 * @param owningObject config object on which to set attributes
	 * @param attrList List of attributes to set
	 */
	public void setConfigAttributes(ObjectName owningObject, AttributeList attrList) ;
	
	/**
	 * save whatever changes have been made
	 * @param overrideConflicts override conflicting changes in the objects
	 */
	public void saveConfig(boolean overrideConflicts) throws ConfigServiceException, ConnectorException ;
	
	/**
	 * discard any changes that have been made in the config.
	 */
	public void discardConfig() throws ConfigServiceException, ConnectorException ;

	/**
	 * resolve the value of a variable
	 * @param curVar Name of the variable
	 * @param scopeObj Scope within the model (cell/node/server)
	 * @return resolved var value
	 */
	public String resolveVar(String curVar, ObjectName scopeObj) ;

	
}