//%Z% %I% %W% %G% %U% [%H% %T%]
/*
 * Change History:
 *
 * Reason         Version    Date       User id     Description
 * ----------------------------------------------------------------------------
 * F017049-19395	8.0		07/21/2010	mcasile		Add remote reader test on HPEL mBean via JMX.
 */

package com.ibm.sample.hpel ;

import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import com.ibm.websphere.management.AdminClient;
import com.ibm.websphere.management.AdminClientFactory;
import com.ibm.websphere.management.Session;
import com.ibm.websphere.management.configservice.ConfigService;
import com.ibm.websphere.management.configservice.ConfigServiceHelper;
import com.ibm.websphere.management.configservice.ConfigServiceProxy;
import com.ibm.websphere.management.exception.ConfigServiceException;
import com.ibm.websphere.management.exception.ConnectorException;

/**
 * The <code>JmxClient</code> class is used to allow client side JUnit tests to drive
 * methods on MBeans to complete server side testing.
 * 
 * @author mcasile
 */
public class JmxClientImpl implements JmxClient {
	private com.ibm.websphere.management.AdminClient adminClient = null ;         // JMX Admin Client
	private static final String sThisClass = JmxClientImpl.class.getName() ;
	private static Logger sJmxClientLogger = Logger.getLogger(sThisClass) ;
	private ConfigService configService = null ;                          // ConfigService object
	private Session session = null ;                                      // ConfigService session
	private String ivHostName = null ;
	private String ivPortNum  = null ;
	private String ivConnType = null ;

	/**
	 * construct an impl using the SOAP demographics
	 * @param hostName network address of the host with the server
	 * @param portNum port number to contact (soap port or rmi port)
	 * @param connType connection type (soap or rmi)
	 */
	public JmxClientImpl(String hostName, String portNum, String connType) throws Exception {
		ivHostName = hostName ;
		ivPortNum = portNum ;
		ivConnType = connType ;
	}
	
	/**
	 * construct an impl using no args (not feasible here, used for simplicity flavor)
	 */
	public JmxClientImpl() throws Exception {
		
	}
	
	/**
	 * create an AdminClient for JMX usage using parms from constructor
	 */
	public void createAdminClient() throws Exception {
		// Set up a Properties object for the JMX connector attributes
		Properties connectProps = new Properties();
//		connectProps.setProperty(com.ibm.websphere.management.AdminClient.CONNECTOR_TYPE, ConnType);
		connectProps.setProperty(AdminClient.CONNECTOR_TYPE, ivConnType);
		connectProps.setProperty(AdminClient.CONNECTOR_HOST, ivHostName);
		connectProps.setProperty(AdminClient.CONNECTOR_PORT, ivPortNum);
       
		createAdminClient(connectProps) ;
	}
	
	/**
	 * create an AdminClient for JMX usage using passed in connection properties (generally more complex environments
	 * supported much better with Simplicity)
	 * @param connectionProperties properties needed to connect to remote server through jmx
	 */
	public void createAdminClient(Properties connectionProperties) throws Exception {       
		// Get an AdminClient based on the connectionProperties
		try {
			adminClient = AdminClientFactory.createAdminClient(connectionProperties);
		} catch (ConnectorException e) {
			throw new Exception("createAdminClient caught connector exception: ", e) ;
		}
	}

	/**
	 * check for an MBean and return an array (plus a warning if caller expects one and gets multiple)
	 * @param QueryString String to use in jmx query for active MBeans
	 * @param CkTooMany boolean to decide if multiples are OK
	 */
	public ObjectName [] Ck4MBean(String QueryString, boolean CkTooMany) {
		Set s = null ;
		try {
			ObjectName queryName = new ObjectName(QueryString);
			s = adminClient.queryNames(queryName, null);
			if (s.size() > 1 && CkTooMany) { 
				TooManyObjects(s, QueryString) ; 
			}
		} catch (MalformedObjectNameException e) {
			sJmxClientLogger.logp(Level.SEVERE, sThisClass, "Ck4MBean", 
				"MalformedObjectNameException on Ck4MBean", e) ;
		} catch (ConnectorException e) {
			sJmxClientLogger.logp(Level.SEVERE, sThisClass, "Ck4MBean", 
				"ConnectorException on Ck4MBean", e) ;
		}
		ObjectName [] ObjArray = (ObjectName [])s.toArray(new ObjectName[s.size()]) ;
		return ObjArray ;
	}

	/**
	 * list Information on an MBean (currently writes to the log 
	 */
	public void listMBeanInfo(ObjectName aObjNm) {
		try {
			MBeanInfo MBeanI1 = adminClient.getMBeanInfo(aObjNm) ;
			sJmxClientLogger.logp(Level.INFO, sThisClass, "listMBeanInfo", "Bean: "+aObjNm+" has MBeanInfo: "+MBeanI1) ;
			MBeanOperationInfo [] MBeanOps = MBeanI1.getOperations() ;
			for (int i = 0; i < MBeanOps.length; i++) {
				sJmxClientLogger.logp(Level.INFO, sThisClass, "listMBeanInfo", 
					i+" Op: "+MBeanOps[i].getName()+ " Returns: "+MBeanOps[i].getReturnType()+
					" And Impact: "+MBeanOps[i].getImpact()) ;
				MBeanParameterInfo [] MBeanParms = MBeanOps[i].getSignature() ;
				for (int j = 0; j < MBeanParms.length; j++) {
					sJmxClientLogger.logp(Level.INFO, sThisClass, "listMBeanInfo", 
						"  "+j+" "+MBeanParms[j].getName()+" "+
						MBeanParms[j].getDescription()+" "+MBeanParms[j].getType()) ;
				}
			}
		} catch (Exception e) {
			sJmxClientLogger.logp(Level.SEVERE, sThisClass, "listMBeanInfo", 
					"Error getting MBean Info", e) ;
		}
	}
	
	private void TooManyObjects(Set s, String QueryString) {
		sJmxClientLogger.logp(Level.WARNING, sThisClass, "TooManyObjects", 
			"Oddly, more than one MBean for "+QueryString+
			" here, there are: "+s.size()) ;
		Iterator si = s.iterator() ;
		String ErrorList = "" ;
		while (si.hasNext()) {
	   		ErrorList += ((ObjectName)si.next()).toString()+"\n" ;
		} 
		sJmxClientLogger.logp(Level.WARNING, sThisClass, "TooManyObjects", 
			"Objects: "+ErrorList) ;
	} 

	/* (non-Javadoc)
	 * @see com.ibm.ws.fat.ras.util.IJmxClient#ObjInvoker(javax.management.ObjectName, java.lang.String, java.lang.Object[], java.lang.String[])
	 */
	public Object ObjInvoker(ObjectName aObjName, String ObjOpName, Object [] ObjParams,
		String [] ObjSignature) throws Exception {
		return adminClient.invoke(aObjName, ObjOpName, (Object [])ObjParams, ObjSignature);
	} 

	/**
	 * retrieves an indexed JMX ObjectName from a set
	 * @param aObjSet java.util.Set containing the Objects (from a query)
	 * @param aIndex Index into the set to retrieve the ObjectName
	 * @return ObjectName at that spot in the set
	 */
	public static ObjectName getObjectName(Set aObjSet, int aIndex) {
		ObjectName [] ObjArray = (ObjectName []) aObjSet.toArray() ;
		Iterator ObjIter = aObjSet.iterator() ;
		int CurCount = 0 ;
		while (ObjIter.hasNext()) {
			if (CurCount++ == aIndex) {
				return (ObjectName)ObjIter.next() ;
			}
		}
		return null ;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.ws.fat.ras.util.IJmxClient#getAttribute(javax.management.ObjectName, java.lang.String)
	 */
	public Object getAttribute(ObjectName aObjNm, String aAttrNm) throws Exception {
		return adminClient.getAttribute(aObjNm, aAttrNm) ;
	}

	/* (non-Javadoc)
	 * @see com.ibm.ws.fat.ras.util.IJmxClient#setAttribute(javax.management.ObjectName, javax.management.Attribute)
	 */
	public void setAttribute(ObjectName aObjNm, Attribute aAttrNm) throws Exception {
		adminClient.setAttribute(aObjNm, aAttrNm) ;
		return ;
	}
	
	public com.ibm.websphere.management.AdminClient getAdminClient() {
		return adminClient ;
	}
	
	
	/**************************************************************************************************************************
	 *  Config methods
	 *	create configService and open a session
	 *************************************************************************************************************************/
	public void createConfigService() {
		if (configService == null || session == null) {
			try {
				configService = new ConfigServiceProxy(adminClient);
//				ConfigServiceProxy configService = new ConfigServiceProxy(adminClient) ;
				session = new Session() ;
			} catch (Exception e) {
				sJmxClientLogger.logp(Level.WARNING, sThisClass, "createConfigService", "Exception creating Config Service: "+e);
				configService = null ;
			}
		}
	}

	/***************************************************************************************************************************
	 *	get a configuration object given a type, a key section, and a scoping object (scope and key can be null)
	 *************************************************************************************************************************/
	public ObjectName [] resolveConfigObject(String fullKey) {
		ObjectName [] MyObjs = null ;
		try {
			MyObjs = configService.resolve(session, fullKey) ; 
		} catch (ConfigServiceException e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigObject", "Threw config exception: "+e) ;
		} catch (ConnectorException e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigObject", "Threw connector exception: "+e) ;
		}
		return MyObjs ;
	}

	/***************************************************************************************************************************
	 *	get a configuration object given a type, a key section, and a scoping object (scope and key can be null)
	 *************************************************************************************************************************/
	public ObjectName [] getConfigObject(String type, String key, ObjectName scope) {
		ObjectName [] MyObjs = null ;
		try {
			MyObjs = configService.queryConfigObjects(session, scope, 
				ConfigServiceHelper.createObjectName(null, type, key), null);
		} catch (ConfigServiceException e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigObject", "Threw config exception: "+e) ;
		} catch (ConnectorException e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigObject", "Threw connector exception: "+e) ;
		}
		return MyObjs ;
	}

	/***************************************************************************************************************************
	 *	Get one or more attributes from the configuration
	 *************************************************************************************************************************/
	public Object getConfigAttribute(ObjectName owningObject, String attrName) {
		try {
			Object attrVal = configService.getAttribute(session, owningObject, attrName) ;
			return attrVal ;
		} catch (ConfigServiceException e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigAttribute", "Threw config exception: "+e) ;
		} catch (ConnectorException e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigAttribute", "Threw connector exception: "+e) ;
		}
	   return null ;
	}

	/**
	 * set config attributes on an object
	 * @param owningObject config object on which to set attributes
	 * @param attrList List of attributes to set
	 */
	public void setConfigAttributes(ObjectName owningObject, AttributeList attrList) {
		try {
			configService.setAttributes(session, owningObject, attrList);
		} catch (Exception e) {
			sJmxClientLogger.logp(Level.WARNING, sThisClass, "setConfigAttributes", "Threw exception: "+e) ;
		}
	}

	@Override
	public void saveConfig(boolean overrideConflicts) throws ConnectorException, ConfigServiceException{
		configService.save(session, overrideConflicts) ;
	}

	@Override
	public void discardConfig() throws ConfigServiceException, ConnectorException {
		configService.discard(session) ;
	}

	/*************************************************************************************************************************
	 * With cfgs, resolving variables is ... an opportunity. Currently, this code will not work if the variable
	 * contains embedded vars or resolves to other variables AND ... the variables are not all at the same scope
	 * (ie: server, node, or cell). Exercise for the reader is to find a simple and deterministic way to navigate
	 * up the config tree (instead of down), until the type is cell. So if type is server and var is not found,
	 * search node, then cell.
	 *************************************************************************************************************************/
	public String resolveVar(String curVar, ObjectName scopeObj) {
	   while (curVar.matches(".*\\$\\{.*\\}.*")) {        // Outer loop keeps checking tailored string
		  int strBegin = curVar.indexOf("${") ;
		  int strEnd = curVar.indexOf("}", strBegin) ;
		  String newVarName = curVar.substring(strBegin+2, strEnd - strBegin) ;
		  String newCurVar = (String) getConfigVarValue(scopeObj, newVarName) ;  // Get other env var from args
		  String endOStr = (strEnd < curVar.length()) ? curVar.substring(strEnd+1) : "" ;
		  String begOStr = (strBegin > 0) ? curVar.substring(0,strBegin-1) : "" ;
		  curVar = begOStr + newCurVar + endOStr ;
	   }
	   return curVar ;
	}

	/**********************************************************************************************************************
	 * get the configuration variable for the variable (which has now been stripped of its trimmings).
	 * @param scopeObj object where variables will be searched
	 * @param varName name of variable to resolve to a value
	 * @return resolved value
	 *********************************************************************************************************************/
	private String getConfigVarValue(ObjectName scopeObj, String varName) {
		boolean higherLevels = true ;
		while (higherLevels) {		// Variables may be at server, node, or cell level
			ObjectName [] varObjs = getConfigObject("VariableSubstitutionEntry", null, scopeObj) ;
			try {
				for (int j = 0; j < varObjs.length ; j++) {
					if (varName.equalsIgnoreCase((String)getConfigAttribute(varObjs[j], "symbolicName"))) {     // Found it
						String retVal = (String)getConfigAttribute(varObjs[j], "value") ;
						// If we have variables imbedded, resolve them, otherwise return this
						return retVal ;
					}
				}
			} catch (Exception e) {
				sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigVarValue", "Threw exception: "+e) ;
				return null ;
			}
				// Did not find variable at that scope, see if there are higher scopes and try them
			String configType = scopeObj.getKeyProperty("_Websphere_Config_Data_Type") ;
			String newType = null ;
			if ("Server".equals(configType))  {
				newType = "Node" ;
			} else {
				if ("Node".equals(configType)) {
					newType = "Cell" ;
				} else {
					higherLevels = false ;
				}
			}
			if (higherLevels) {
				try {
					higherLevels = false ;		// Bad news unless we go the one good path
					ObjectName [] newLevel = configService.getRelationship(session, scopeObj, "parent") ;
					if (newLevel == null) {
						sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigVarValue", "No relationship: "+newType+
							"objects for scopeObj") ;
					} else {
						for (ObjectName curObj : newLevel) {
							if (newType.equals(curObj.getKeyProperty("_Websphere_Config_Data_Type"))) {
								higherLevels = true ;
								scopeObj = curObj ;
							}
						}
						sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigVarValue", "Object: "+scopeObj+" has no relations of type: "+newType) ;
					}
				} catch (ConfigServiceException e) {
					sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigVarValue", "Threw config exception: "+e) ;
				} catch (ConnectorException e) {
					sJmxClientLogger.logp(Level.WARNING, sThisClass, "getConfigVarValue", "Threw connector exception: "+e) ;
				}
			}
		}
		return null ;
	}

	/**
	 * generate an attributeList if the specified key warrants it
	 * @param cfgKey the key string, if it has attributes, use them
	 * @return An AttributeList .. or null
	 */
	private AttributeList generateAttributesFromKey(String cfgKey, String type) {
		if (cfgKey == null)  return null ;
		if (cfgKey.startsWith("/") && cfgKey.endsWith("/")) {
			AttributeList newAttrList = new AttributeList() ;
			String [] clauses = cfgKey.split("/") ;
			for (int i = 1; i < clauses.length; i++) {	// Ignore starting slash 
				int colonPos = clauses[i].indexOf(':') ;
				System.out.println("AttrNm: "+clauses[i].substring(0, colonPos)+" AttrVal: "+clauses[i].substring(colonPos+1));
				newAttrList.add(new Attribute(clauses[i].substring(0, colonPos), clauses[i].substring(colonPos+1))) ;
			}
			return newAttrList ;
		} else {
			return null ;
		}
	}
}