package com.ibm.sample.hpel;

import java.util.Date;
import java.util.logging.Level;

import com.ibm.websphere.logging.hpel.reader.LogQueryBean;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryException;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryRuntimeException;
import com.ibm.websphere.logging.hpel.reader.MergedRepository;
import com.ibm.websphere.logging.hpel.reader.RepositoryLogRecord;
import com.ibm.websphere.logging.hpel.reader.RepositoryReader;
import com.ibm.websphere.logging.hpel.reader.RepositoryReaderImpl;
import com.ibm.websphere.logging.hpel.reader.ServerInstanceLogRecordList;

/**
 * This uses the simple read API to pull from n different log repositories on this system, and merges the data.  This needs only
 * the com.ibm.hpel.logging.jar.  The printout is just enough to show it worked, but a wealth of data is available as simple
 * get methods on the RepositoryLogRecords and the headers.  This pulls just warning and higher, but note that the getLogLists
 * and getLogListForServerInstance take the LogQueryBean object as a parm and this enables extensive filtering.  This sample
 * marges only the latest logs for each server (ie: the logs associated with the latest server instance).  A server instance
 * is one server lifecycle which means from start to stop of a server.  For z/OS, this is start to stop of the controller.
 * On z/OS, there are child processes, and these are also merged in this scenario. 
 */
public class LocalMergeSample {
			// One argument per repository base, each will be merged. Example of arg1:  /opt/IBM/WasX/profiles/AppSrv01/logs/server1
			//  and arg2 /opt/IBM/WasX/profiles/ApSrv02/logs/server2
	public static void main(String[] args) {
		ServerInstanceLogRecordList [] silrlList = new ServerInstanceLogRecordList[args.length] ;
				// logQueryBean has many filtering options which can be combined for complex queries
		LogQueryBean logQueryBean = new LogQueryBean() ;
		logQueryBean.setLevels(Level.WARNING, Level.SEVERE) ;
		
		for (int i = 0; i < args.length; i++) {		// For each repository directory
					// Create a repository reader (requires base directory of repository 
			RepositoryReader logRepository = new RepositoryReaderImpl(args[i]) ;		
					// Pull from just the current instance (null date) of the server for merging on appropriate times
					// Could pull from all or use filter criteria that would select several, this is for simplicity
			try {
						// Read records from the current server instance in repository that match query bean
				silrlList[i] = logRepository.getLogListForServerInstance((Date)null, logQueryBean) ;
			} catch (LogRepositoryException e) {
				System.out.println("Exception on getLogListForServerInstance: "+e);
				System.exit(-99) ;
			}
		}
					// Merge records from all resultSets contained in the array
		MergedRepository result = new MergedRepository(silrlList) ;  	// Merge the current serverInstance from each server
		System.out.println("ServerNm/Pid    Date/Time    Thread    Msg");
		try {
			for (RepositoryLogRecord repositoryLogRecord : result) {		// For each record in new merged set
					// Note that each record references back to the header for that file to access key info there
				String serverNm = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_SERVER_NAME) ;
				String pid = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_PROCESSID) ;
					// Just printing some key information here.  Note that the repositoryRecord exposes all fields
					// with simple get methods and gives access to the header information associated with each as well
				System.out.println(serverNm+"/"+pid+"  "+new Date(repositoryLogRecord.getMillis())+"  "+
					repositoryLogRecord.getThreadID()+"  "+repositoryLogRecord.getFormattedMessage());
			}
		} catch (LogRepositoryRuntimeException lrre) {
			System.out.println("reading merged repository threw LogRepositoryRuntimeException: "+lrre);
		}
	}
}