package com.ibm.sample.hpel;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

import com.ibm.websphere.logging.hpel.reader.LogQueryBean;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryException;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryRuntimeException;
import com.ibm.websphere.logging.hpel.reader.MergedRepository;
import com.ibm.websphere.logging.hpel.reader.RepositoryLogRecord;
import com.ibm.websphere.logging.hpel.reader.RepositoryReader;
import com.ibm.websphere.logging.hpel.reader.RepositoryReaderImpl;
import com.ibm.websphere.logging.hpel.reader.ServerInstanceLogRecordList;

/**
 * This uses the simple read API to pull from n different log repositories on this system, and merges the data.  This needs only
 * the com.ibm.hpel.logging.jar.  The printout is just enough to show it worked, but a wealth of data is available as simple
 * get methods on the RepositoryLogRecords and the headers.  This pulls just warning and higher, but note that the getLogLists
 * and getLogListForServerInstance take the LogQueryBean object as a parm and this enables extensive filtering.  This sample
 * marges only the latest logs for each server (ie: the logs associated with the latest server instance).  A server instance
 * is one server lifecycle which means from start to stop of a server.  For z/OS, this is start to stop of the controller.
 * On z/OS, there are child processes, and these are also merged in this scenario.  This will work against z/OS or distributed
 * (distributed will have no child instances). 
 */
public class LocalMergeSampleZos {
			// One argument per repository base, each will be merged. Example of arg1:  /opt/IBM/WasX/profiles/AppSrv01/logs/server1
			//  and arg2 /opt/IBM/WasX/profiles/ApSrv02/logs/server2
	public static void main(String[] args) {
				// Since lists (on z/OS) can have children, store each found list in an arrayList
		ArrayList <ServerInstanceLogRecordList> silrlList = new ArrayList<ServerInstanceLogRecordList>() ;
				// logQueryBean has many filtering options which can be combined for complex queries
		LogQueryBean logQueryBean = new LogQueryBean() ;
		logQueryBean.setLevels(Level.WARNING, Level.SEVERE) ;
		
		for (int i = 0; i < args.length; i++) {		// For each repository directory
					// Create a repository reader (requires base directory of repository 
			RepositoryReader logRepository = new RepositoryReaderImpl(args[i]) ;		
					// Pull from just the current instance (null date) of the server for merging on appropriate times
					// Could pull from all or use filter criteria that would select several, this is for simplicity
			try {
						// Read records from the current server instance in repository that match query bean
				ServerInstanceLogRecordList silrl = logRepository.getLogListForServerInstance((Date)null, logQueryBean) ;
						// Add resultSet to ArrayList
				silrlList.add(silrl) ;
						// Look for child resultSets (ie: z/OS servants and adjuncts)
		    	Map<String, ServerInstanceLogRecordList>servantMap = silrl.getChildren() ;
		    			// Each child has its own resultSet, add each resultSet to the arrayList (no affect on distributed)
		        Iterator<String> servantKeys = servantMap.keySet().iterator() ;
		        while (servantKeys.hasNext()) {
		        	String label = servantKeys.next() ;			// Map label has key information to identify which child process
		        	ServerInstanceLogRecordList servantRecs = servantMap.get(label) ;		// Pull the child process serverInstance
		        	silrlList.add(servantRecs) ;
		        }
			} catch (LogRepositoryException e) {
				System.out.println("Exception on getLogListForServerInstance: "+e);
				System.exit(-99) ;
			}
		}
					// Convert the arrayList into an array
		ServerInstanceLogRecordList [] silrlArray = new ServerInstanceLogRecordList[silrlList.size()] ;
		silrlArray = silrlList.toArray(silrlArray) ;
					// Merge records from all resultSets contained in the array
		MergedRepository result = new MergedRepository(silrlArray) ;  	// Merge the current serverInstance from each server
		System.out.println("ServerNm/Pid    Date/Time    Thread    Msg");
		try {
			for (RepositoryLogRecord repositoryLogRecord : result) {		// For each record in new merged set
					// Note that each record references back to the header for that file to access key info there
				String serverNm = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_SERVER_NAME) ;
				String pid = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_PROCESSID) ;
					// Just printing some key information here.  Note that the repositoryRecord exposes all fields
					// with simple get methods and gives access to the header information associated with each as well
				System.out.println(serverNm+"/"+pid+"  "+new Date(repositoryLogRecord.getMillis())+"  "+
					repositoryLogRecord.getThreadID()+"  "+repositoryLogRecord.getFormattedMessage());
			}
		} catch (LogRepositoryRuntimeException lrre) {
			System.out.println("Retrieving rows threw LogRepositoryRuntimeException: "+lrre);
		}
	}
}