package com.ibm.sample.hpel;

import java.util.logging.Level;

import com.ibm.websphere.logging.hpel.reader.LogQueryBean;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryException;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryRuntimeException;
import com.ibm.websphere.logging.hpel.reader.RepositoryLogRecord;
import com.ibm.websphere.logging.hpel.reader.RepositoryPointer;
import com.ibm.websphere.logging.hpel.reader.RepositoryReader;
import com.ibm.websphere.logging.hpel.reader.RepositoryReaderImpl;
import com.ibm.websphere.logging.hpel.reader.ServerInstanceLogRecordList;

/**
 * simple sample of using the Read API.  To make this run requires only the com.ibm.hpel.logging.jar in the buildpath
 * and runtime classpath. In practice, more information will be printed for each record and special formatting will be done,
 * this is primarily a sample to enable someone to begin using the technology with a simple self-contained program.  It goes
 * through all ServerInstances in a repository.  It does not handle a z/OS condition of child processes.  See other samples
 * for that. The polling loop is what someone could do to monitor the logs (ie: come back every n minutes or n seconds and
 * find the new data)
 */
public class LocalRadSample {

	public static void main(String[] args) {
			// Assumes strings properly passed in, since real usage will be in different environment
			// My first run used args: /home/mcasile/AppX/logs/server1 and 5
		String repositoryDirectory = args[0] ;
		int pollingInterval = Integer.parseInt(args[1]) ;	// To avoid burning too many cycles, at least 3 second delay recommended
		// Create a repository reader (requires base directory of repository 
		RepositoryReader logRepository = new RepositoryReaderImpl(repositoryDirectory) ;		
				// Get iterator of server instances (start/stop of the server) extracting all log messages with
				// severity between INFO and SEVERE.  Lots of different filtering options, this is just one sample
		RepositoryPointer repositoryPointer = null ;	// This will be used as a our last record shown
		ServerInstanceLogRecordList repResults = null;	// Since only looking at current server, only one SILRL
		LogQueryBean logQueryBean = new LogQueryBean() ;		// This bean allows many filtering options
		logQueryBean.setLevels(Level.INFO, Level.SEVERE) ;	// We're looking at WARNING and SEVERE here, but can use any
		while (true) {		// Timing loop
			try {
				repResults = logRepository.getLogListForServerInstance(repositoryPointer, logQueryBean) ;
			} catch (LogRepositoryException e) {
				System.out.println("Exception in get log lists: "+e);
			}
			RepositoryPointer newPointer = repositoryPointer ;
				// Go through the records for this (the  most current) server instance
			try {
				for (RepositoryLogRecord repositoryLogRecord : repResults) {
					// Just printing some key information here.  Note that the repositoryRecord exposes all fields
					// with simple get methods
					System.out.println("  "+repositoryLogRecord.getFormattedMessage());
					newPointer = repositoryLogRecord.getRepositoryPointer() ;	// Brute force method to get last one	
				}
			} catch (LogRepositoryRuntimeException lrre) {
				System.out.println("Retrieving rows threw LogRepositoryRuntimeException: "+lrre);
			}
			if (newPointer != null)  repositoryPointer = newPointer ;
			try { Thread.sleep(pollingInterval * 1000) ; } catch (Exception e) {} ;
		}
	}
}