package com.ibm.sample.hpel;

import java.util.Locale;

import javax.management.ObjectName;

		// AbstractRemoteRepositoryReader handles the primary logic, but subClass (this class) provides jmx comms)
import com.ibm.websphere.logging.hpel.reader.AbstractRemoteRepositoryReader;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryException;
		// The LogQueryBean is used to pass the core information about the query through the superClass
import com.ibm.websphere.logging.hpel.reader.LogQueryBean;
		// Bringing back results from multiple server instaances (starts/stops of a server)
import com.ibm.websphere.logging.hpel.reader.RemoteAllResults;
		// Information to go back to a current server instance
import com.ibm.websphere.logging.hpel.reader.RemoteInstanceDetails;
		// Result for a single server instance
import com.ibm.websphere.logging.hpel.reader.RemoteInstanceResult;
		// Pointer inside a repository where prior call left off
import com.ibm.websphere.logging.hpel.reader.RepositoryPointer;

/*
 * subClass of AbstractRemoteRepositoryReader that provides JMX communication
 */
public class RemoteReaderHelper extends AbstractRemoteRepositoryReader {
	
	private JmxClient jmxClient = null ;		// JMX Client to connect to a remote server
	private String  ivServerContext = null ;
	
	private ObjectName hpelControlBean = null ;		// Reference to the hpelControl MBean for a server

	/**
	 * constructor which sets up for subsequent calls (connects to jmx et al
	 * @param cellName
	 * @param nodeName
	 * @param serverName
	 * @param hostName
	 * @param portNum
	 * @param connType
	 */
	public RemoteReaderHelper(String cellName, String nodeName, String serverName, String hostName,
		String portNum, String connType) throws Exception {
		this(cellName, nodeName, serverName, hostName, portNum, connType, null) ;
	}
	
	public RemoteReaderHelper(String cellName, String nodeName, String serverName, String hostName,
		String portNum, String connType, String serverContext) throws Exception {
		super(Locale.getDefault()) ;
		System.out.println("Starting test w/cell: "+cellName+" node: "+nodeName+" server: "+serverName);
		System.out.println("  host: "+hostName+" port: "+portNum+" connType: "+connType+" ServerContext: "+serverContext);
		ivServerContext = serverContext ;

		jmxClient = new JmxClientImpl(hostName, portNum, connType) ;
		if (jmxClient != null) {
			jmxClient.createAdminClient() ;

			ObjectName [] hpelControlBeans = 
				jmxClient.Ck4MBean("WebSphere:cell="+cellName+",node="+nodeName+",type=HPELControlService,process="+serverName+",*", true) ;
			hpelControlBean = (hpelControlBeans == null) ? null : hpelControlBeans[0] ;
		}
	}
		
	@Override
	public RemoteAllResults readLogLists(LogQueryBean query, RepositoryPointer after) {
		RemoteAllResults remoteAllResults = null ;
		try {
			remoteAllResults = (RemoteAllResults)jmxClient.ObjInvoker(hpelControlBean, "getLogLists", 
				new Object [] { query, after, ivServerContext}, 
				new String [] {"com.ibm.websphere.logging.hpel.reader.LogQueryBean", 
				"com.ibm.websphere.logging.hpel.reader.RepositoryPointer", "java.lang.String" });
		} catch (Exception e) {
			System.out.println("Exception in reading log lists: "+e);
			e.printStackTrace();
		}
		return remoteAllResults ;
	}

	@Override
	public RemoteInstanceResult readLogRecords(RemoteInstanceDetails indicator,
			RepositoryPointer after, int offset, int size, Locale locale)
			throws LogRepositoryException {
		RemoteInstanceResult remoteInstanceResult = null;
		try {
			remoteInstanceResult = (RemoteInstanceResult)jmxClient.ObjInvoker(hpelControlBean,
					"getLogListForServerInstance", new Object [] { indicator, after, offset, size, locale, ivServerContext}, 
					new String [] {"com.ibm.websphere.logging.hpel.reader.RemoteInstanceDetails",
						"com.ibm.websphere.logging.hpel.reader.RepositoryPointer", "int", "int", "java.util.Locale", "java.lang.String"});
		} catch (Exception e) {
			System.out.println("Exception in reading log records: "+e);
			e.printStackTrace();
		}
		return remoteInstanceResult;
	}
}
