package com.ibm.sample.hpel;

import java.util.Date;
import java.util.logging.Level;

import com.ibm.websphere.logging.hpel.reader.LogRepositoryException;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryRuntimeException;
import com.ibm.websphere.logging.hpel.reader.RepositoryLogRecord;
import com.ibm.websphere.logging.hpel.reader.ServerInstanceLogRecordList;

/**
 * simple sample of using the Remote Read API.  To make this run requires the com.ibm.hpel.logging.jar and the 
 * com.ibm.ws.admin.client_8.0.0.jar in the buildpath and runtime classpath
 * -Dcom.ibm.SOAP.ConfigURL=file:/tmp/WasX/profiles/AppSrv01/properties/soap.client.props
 * -Dcom.ibm.SSL.ConfigURL=file:/tmp/WasX/profiles/AppSrv01/properties/ssl.client.props
 * -Dcom.ibm.ffdc.log=/tmp/ffdcJmx/
 */
public class RemoteReaderSample {

	public static void main(String[] args) {
				// Create simple reader object with the WebSphere topology and network topology info
		if (args.length < 6) {
			System.out.println("Usage is: java RemoteReaderSample <cellName> <nodeName> <serverName> <hostName> <hostPort> <connectionType>");
			System.out.println(" ie: java RemoteReaderSample mycell mynode server1 localhost 8880 soap");
			System.exit(-1) ;
		}
        System.out.println("Using: Cell: "+args[0]+" node: "+args[1]+" server: "+args[2]+" host: "+args[3]+
            " port: "+args[4]+" connType: "+args[5]) ;
		RemoteReaderHelper simpleRemoteReader = null ;
		try {
			simpleRemoteReader = new RemoteReaderHelper(args[0], args[1], args[2], 
				args[3], args[4], args[5]) ;
		} catch (Exception e) {
			System.out.println("Exception in constructing RemoteReaderHelper: "+e);
			e.printStackTrace() ;
			System.exit(0) ;
		}

				// Use remote reader and get list of all Warning and Errors from the current server instance of
				// the remote server. Can go across server instances, and numerous filtering options are available.
				// For remote API, it is important to filter in this call where possible to reduce the # of rows from
				// the remote log that come back.
		ServerInstanceLogRecordList silrl = null ;
		try {
			silrl = simpleRemoteReader.getLogListForServerInstance((Date)null, Level.INFO, 
				Level.SEVERE);
		} catch (LogRepositoryException e) {
			System.out.println("Data retrieval resulted in an exception: "+e);
			e.printStackTrace();
			System.exit(-99) ;
		}
		try {
			for (RepositoryLogRecord logRec : silrl) {
				System.out.println(logRec.getFormattedMessage());
			}
		} catch (LogRepositoryRuntimeException lrre) {
			System.out.println("Runtime Exception caught retrieving log repository data: "+lrre);
		}
	}
}